$:.unshift File.expand_path("../../lib", __FILE__)
# $:.unshift File.expand_path("../../test", __FILE__)

require "test/unit"
require "thrift_client"
require "yaml"
require "feed"
require 'em-synchrony'
require 'em-synchrony/fiber_iterator'
require 'stat-analysis-query'

class ClientTest < Test::Unit::TestCase
  include Test::Unit::Assertions
  def setup
    super
    root = File.expand_path("../..", __FILE__)
    @config = YAML.load_file(File.join(root, 'test/thrift_config.yaml'))
    puts "setup"
  end

  def test_thread_client
    client = ThriftClient.new(@config['client_feed'])
    while 1
      Thread.new {
        client.getWebUnread('3428')
      }
      sleep(0.01)
    end
  end

  def test_fiber_client
    EM.synchrony do
      client = ThriftClient.new(@config['client_em_feed'])
      start = Time.now
      number = 0
      while number < 10
        EM::Synchrony::FiberIterator.new(0...5, 5).each do |i|
          client.getWebSoundRange('12', 0, 100)
          puts i
        end
        number += 1
      end
      stop = Time.now
      em_use = stop - start
      EM.stop
    end
  end

  def test_thread_return_nil
    client = ThriftClient.new(@config['client_query'])
    sleep(3)
    begin
      p client.selectWebHumanCategory
    rescue Exception => e
      backtrace = ""
      e.backtrace.each { | s |
        backtrace.concat(s).concat("\n")
      }
      # puts "#{e} #{e.backtrace}"
      puts "#{e.class} #{e.message} #{backtrace}"
    end
    sleep(1)
    begin
      p client.selectWebAlbumCategoryAndTag
    rescue Exception => e
      puts "#{e} #{e.backtrace}"
    end
    begin
      p client.selectWebAlbumCategoryAndTag
    rescue Exception => e
      puts "#{e} #{e.backtrace}"
    end
  end

  def test_fiber_return_nil
    EM.synchrony do
      client = ThriftClient.new(@config['client_em_query'])
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...1, 1).each do |i|
        begin
          p client.selectWebHumanCategory
        rescue Exception => e
          puts "#{e.class} #{e.message} #{e.backtrace}"
        end
        sleep(1)
        begin
          p client.selectWebAlbumCategoryAndTag
        rescue Exception => e
          backtrace = ""
          e.backtrace.each { | s |
            backtrace.concat(s).concat("\n")
          }
          puts "#{e} #{e.backtrace}"
        end
        begin
          p client.selectWebAlbumCategoryAndTag
        rescue Exception => e
          puts "#{e} #{backtrace}"
        end
      end
      stop = Time.now
      em_use = stop - start
      EM.stop
    end
  end

  def teardown
    super
    puts "teardown"
  end
end
