$:.unshift File.expand_path("../../lib", __FILE__)
$:.unshift File.expand_path("../../test", __FILE__)

require "test/unit"
require "thrift_client"
require 'thrift'
require 'yaml'
require "foobar/handler"
require "helper"
require 'em-synchrony'
require 'em-synchrony/fiber_iterator'

class EventMachineTransprotTest < Test::Unit::TestCase
  include Test::Unit::Assertions
  def setup
    super
    root = File.expand_path("../..", __FILE__)
    @config = YAML.load_file(File.join(root, 'test/thrift_config.yaml'))
    puts "setup"
  end

  def test_eventmachine_transport_performence
    em_use = 0
    no_em_use = 0
    EM.synchrony do
      client = ThriftClient.new(@config['em_test'])
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...100, 50).each do |i|
        client.wait(1)
      end
      stop = Time.now
      em_use = stop - start
      EM.stop
    end
    EM.synchrony do
      client = ThriftClient.new(@config['no_em_test'])
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...100, 50).each do |i|
          client.wait(1)
      end
      stop = Time.now
      no_em_use = stop - start
      EM.stop
    end
    # assert(em_use < no_em_use)
    puts "eventmachine use #{em_use}"
    puts "no eventmachine use #{no_em_use}"
  end

  def test_eventmachine_transport_availability
    EM.synchrony do
      client = ThriftClient.new(@config['em_availability_test'])
        client.getBar
        client.getBar
      EM.stop
    end
    EM.synchrony do
      client = ThriftClient.new(@config['em_availability_test'])
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...1000, 50).each do |i|
        puts "do"
        bar = Test::Bar.new
        bar.a = i.to_s
        bar.b = i
        bar.c = true
        bar.d = i.to_f
        bar.e = i
        ret = client.putBar bar
        assert(ret)
      end
      stop = Time.now
      em_use = stop - start
      puts em_use
      EM.stop
    end

    EM.synchrony do
      client = ThriftClient.new(@config['em_availability_test'])
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...1000, 50).each do |i|
        bar = client.getBar
        assert_equal("1111111111",bar.a)
        assert_equal(11111111,bar.b)
        assert_equal(false,bar.c)
        assert_equal(1111111.232323,bar.d)
        assert_equal(111111111111111,bar.e)
      end
      stop = Time.now
      em_use = stop - start
      puts em_use
      EM.stop
    end
  end

  def test_eventmachine_reconnect
    puts "should start new server on port 9001"
    # sleep(5)
    EM.synchrony do
      client = ThriftClient.new(@config['em_availability_test'])

      assert_nothing_raised do
        bar = client.getBar
        puts bar.inspect
        assert_equal("1111111111",bar.a)
        assert_equal(11111111,bar.b)
        assert_equal(false,bar.c)
        assert_equal(1111111.232323,bar.d)
        assert_equal(111111111111111,bar.e)
      end

      puts "should shutdown new server on port 9001"
      sleep(3)

      assert_raise Thrift::TransportException do
        client.getBar
      end

      puts "should start new server on port 9001"
      sleep(5)

      assert_nothing_raised do
        client.getBar
      end
    EM.stop
    end
  end

  def test_timer
    puts "should start new server on port 9001"
    # sleep(5)
    EM.synchrony do
      client = ThriftClient.new(@config['em_test'])
      i = 0
      # EM::Synchrony::FiberIterator.new(0...1000, 1000).each do |i|
        client.timeout
      # end
      EM.stop
    end
  end

  def teardown
    super
    # Test::Helper.shutdown_server
    puts "teardown"
  end
end
