$:.unshift File.expand_path("../../lib", __FILE__)
$:.unshift File.expand_path("../../test", __FILE__)

require "test/unit"
require 'thrift'
require "thrift_client"
require "foobar/handler"
require "helper"
require 'yaml'

class MultiplexedTest < Test::Unit::TestCase
  include Test::Unit::Assertions
  def setup
    super
    root = File.expand_path("../..", __FILE__)
    @config = YAML.load_file(File.join(root, 'test/thrift_config.yaml'))
    Test::Helper.start_server
    puts "setup"
  end

  def test_multiplexed
    client = ThriftClient.new(@config['multiplexed_test'])
    puts "test start"

    foo = client.getFoo
    assert_equal(200000, foo.first)
    assert_equal(222222222222222, foo.second)
    assert_equal(2.2, foo.third)
    assert_equal("222", foo.fourth)

    assert_raise do
      client.ping
    end
    puts "test stop"
  end

  def test_error_multiplexed
    client = ThriftClient.new(@config['multiplexed_error_test'])
    assert_raise do
      client.getFoo
    end
  end

  def test_profile
    client = ThriftClient.new({"transport" => "eventmachine","size"=>50,"client_class" => "Passport::Thrift::RemoteUserProfileService::Client","servers" => "192.168.1.173:9100"})
    EM.synchrony do
      start = Time.now
      EM::Synchrony::FiberIterator.new(0...100, 50).each do |i|
        client.queryUserBasicInfo(3033)
      end
      stop = Time.now
      em_use = stop - start
      EM.stop
    end
  end

  def teardown
    super
    Test::Helper.shutdown_server
    puts "teardown"
  end
end
